package ru.ulstu.extractor.ts.creator.db;

import org.springframework.stereotype.Component;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.branch.service.BranchService;
import ru.ulstu.extractor.commit.model.Commit;
import ru.ulstu.extractor.commit.service.CommitService;
import ru.ulstu.extractor.gitrepository.model.FileChange;
import ru.ulstu.extractor.gitrepository.model.LineChange;
import ru.ulstu.extractor.heuristic.api.StructuralUnitIdentifier;
import ru.ulstu.extractor.ts.model.TimeSeries;
import ru.ulstu.extractor.ts.model.TimeSeriesType;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;
import ru.ulstu.extractor.ts.service.TimeSeriesService;

import java.util.ArrayList;
import java.util.List;

@Component
public class ClassTS extends DBTimeSeriesCreator {
    private final TimeSeriesService timeSeriesService;
    private final CommitService commitService;
    private final BranchService branchService;
    private final StructuralUnitIdentifier structuralUnitIdentifier;

    public ClassTS(TimeSeriesService timeSeriesService,
                   CommitService commitService,
                   BranchService branchService,
                   StructuralUnitIdentifier structuralUnitIdentifier) {
        this.timeSeriesService = timeSeriesService;
        this.commitService = commitService;
        this.branchService = branchService;
        this.structuralUnitIdentifier = structuralUnitIdentifier;
    }

    @Override
    public List<TimeSeries> getTimeSeries(Branch branch) {
        List<TimeSeries> timeSeriesResult = new ArrayList<>();
        List<Commit> commits = new ArrayList<>(commitService.findByRepositoryIdAndName(branch.getGitRepository().getId(),
                branch.getName()));
        TimeSeries timeSeries = new TimeSeries(
                String.format("%s %s %s",
                        branch.getGitRepository().getName(),
                        branch.getName(),
                        getTimeSeriesType().getDescription()),
                branchService.findByRepositoryAndName(branch.getGitRepository(),
                        branch.getName()),
                getTimeSeriesType());
        for (Commit commit : commits) {
            double value = 0;
            for (FileChange fileChange : commit.getFileChanges()) {
                for (LineChange lineChange : fileChange.getLineChanges()) {
                    if (lineChange.getLineTo() != null && !lineChange.getLineTo().isEmpty()) {
                        value += structuralUnitIdentifier.getClasses(lineChange.getLineTo()).size();
                    }
                }
            }
            timeSeries.getValues().add(new TimeSeriesValue(commit.getDate(), value));
        }
        if (!timeSeries.getValues().isEmpty()) {
            timeSeriesResult.add(timeSeries);
        }
        return timeSeriesResult;
    }

    @Override
    public TimeSeriesService getTimeSeriesService() {
        return timeSeriesService;
    }

    @Override
    public TimeSeriesType getTimeSeriesType() {
        return TimeSeriesType.CLASSES;
    }
}
